/*
 * QEstEidCommon
 *
 * Copyright (C) 2009-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2009-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include <QtCore/QtGlobal>

#ifdef Q_OS_MAC
#include <QtGui/QApplication>
#define BaseApplication QApplication
#else
#include "qtsingleapplication/src/QtSingleApplication"
#define BaseApplication QtSingleApplication
#endif

#include <QtCore/QEvent>
#include <QtCore/QStringList>

struct ApplicationStruct;
class QLabel;
class QSslCertificate;
class QUrl;

class Common: public BaseApplication
{
	Q_OBJECT
public:
	Common( int &argc, char **argv );
	virtual ~Common();

	virtual QString lastPath() const { return QString(); }
	virtual void setLastPath( const QString & ) {}
	virtual bool event( QEvent *e );

	static QString applicationOs();
	static bool cardsOrder( const QString &s1, const QString &s2 );
	static QString fileSize( quint64 bytes );
	static QUrl helpUrl();
	static QStringList packages( const QStringList &names, bool withName = true );
	static bool runPrivileged( const QString &program, const QStringList &arguments );
	static void setAccessibleName( QLabel *widget );
	static void showHelp( const QString &msg, int code = -1 );
	static bool startDetached( const QString &program, const QStringList &arguments = QStringList() );
	static QString tempFilename();

public Q_SLOTS:
	void browse( const QUrl &url );
	void mailTo( const QUrl &url );

private:
	static quint8 cardsOrderScore( QChar c );

#if defined(Q_OS_MAC)
	void initMacEvents();
	void deinitMacEvents();
	ApplicationStruct *macEvents;
#endif
};

class REOpenEvent: public QEvent
{
public:
	enum { Type = QEvent::User + 1 };
	REOpenEvent(): QEvent( QEvent::Type(Type) ) {}
};

class OpenFilesEvent: public QEvent
{
public:
	enum { Type = QEvent::User + 2 };
	OpenFilesEvent( const QStringList &files ): QEvent( QEvent::Type(Type) ), m_files(files) {}

	QStringList files() const { return m_files; }

private:
	QStringList m_files;
};
