/*
 * QEstEidCommon
 *
 * Copyright (C) 2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "DateTime.h"

#include <QtCore/QLocale>

DateTime::DateTime( const QDateTime &other, bool adjustZone )
	: QDateTime( adjustZone ? other.addSecs( timeDiff() ) : other )
{}

QString DateTime::formatDate( const QString &format ) const
{
	int pos = format.indexOf( "MMMM" );
	if( pos == -1 )
		return toString( format );
	QString d = toString( QString( format ).remove( pos, 4 ) );
	return d.insert( pos, QLocale().monthName( date().month() ) );
}

int DateTime::timeDiff() const
{
	QDateTime utc = QDateTime::currentDateTime().toUTC();
	utc.setTimeSpec( Qt::LocalTime );
	return utc.secsTo( QDateTime::currentDateTime() );
}

QString DateTime::toStringZ( const QString &format ) const
{
	if( !isValid() ) return "";
	int diffsec = timeDiff();
	return toString( format ) +	QString(" %1%2").arg( diffsec >= 0 ? "+" : "-" )
		.arg( QTime().addSecs( diffsec >= 0 ? diffsec : -diffsec ).toString( "hh:mm" ) );
}
