/*
 * QEstEidCommon
 *
 * Copyright (C) 2009-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2009-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#include "Diagnostics.h"

#include "QPCSC.h"

#include <QtCore/QStringList>
#include <QtCore/QTextStream>
#include <QtNetwork/QSslCertificate>

Diagnostics::Diagnostics()
	: QThread( 0 )
{
	moveToThread( this );
}

QString Diagnostics::getReaderInfo() const
{
	QString d;
	QTextStream s( &d );

	QPCSC manager;
	s << "<b>" << tr("Smart Card service status: ") << "</b>" << " "
		<< (manager.serviceRunning() ? tr("Running") : tr("Not running"));

	s << "<br /><b>" << tr("Card readers") << ":</b><br />";
	foreach( const QString &reader, manager.readers() )
	{
		s << reader;
		QPCSCReader atr( reader, &manager );
		if( reader.contains( "EZIO SHIELD", Qt::CaseInsensitive ) )
		{
			s << " - Secure PinPad";
			if( !atr.isPinPad() )
				s << " (Driver missing)";
		}
		else if( atr.isPinPad() )
			s << " - PinPad";

		if( !atr.isPresent() )
		{
			s << "<br />";
			continue;
		}

		atr.connect();
		atr.reconnect( QPCSCReader::UnpowerCard );
		QString cold = atr.atr();
		atr.reconnect( QPCSCReader::ResetCard );
		QString warm = atr.atr();

		atr.transfer( QByteArray("\x00\xA4\x00\x0C", 4) ); // MASTER FILE
		atr.transfer( QByteArray("\x00\xA4\x01\x0C\x02\xEE\xEE", 7) ); // ESTEID DATAFILE
		atr.transfer( QByteArray("\x00\xA4\x02\x04\x02\x50\x44", 7) ); // PERSONAL DATAFILE
		QString id = atr.transfer( QByteArray("\x00\xB2\x07\x04\x00", 5) ); // read card id

		if( id.trimmed().isEmpty() )
		{
			atr.transfer( QByteArray("\x00\xA4\x02\x04\x02\xAA\xCE", 7) );
			QByteArray cert;
			while( cert.size() < 0x0600 )
			{
				unsigned char cmd[] = { 0x00, 0xB0, cert.size() >> 8, cert.size(), 0x00 };
				QByteArray data = atr.transfer( QByteArray( (char *)&cmd, sizeof(cmd) ) );
				if( data.isEmpty() )
					break;
				cert += data;
			}
			id = QSslCertificate( cert, QSsl::Der ).subjectInfo( "serialNumber" );
		}

		s << "<p style='margin-left: 10px; margin-top: 0px; margin-bottom: 0px; margin-right: 0px;'>"
		  << "ID - " << id << "<br />ATR cold - " << cold << "<br />ATR warm - " << warm << "</p>";
	}
	if( manager.readers().isEmpty() )
		s << tr("No readers found");

#ifdef Q_OS_WIN
	s << "<b>" << tr("Card drivers") << ":</b><br />" << manager.drivers().join( "<br />" );
#endif

	return d;
}
