/*
 * QEstEidCommon
 *
 * Copyright (C) 2011-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2011-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include <QtCore/QObject>

class QPCSCReader;
class QPCSCPrivate;
class QPCSC: public QObject
{
	Q_OBJECT
public:
	explicit QPCSC( QObject *parent = 0 );
	~QPCSC();

	QStringList drivers() const;
	QStringList readers() const;
	bool serviceRunning() const;

private:
	QPCSCPrivate *d;

	friend class QPCSCReader;
};

class QPCSCReaderPrivate;
class QPCSCReader: public QObject
{
	Q_OBJECT
public:
	enum Reset
	{
		LeaveCard = 0,
		ResetCard = 1,
		UnpowerCard = 2,
		EjectCard = 3
	};

	explicit QPCSCReader( const QString &reader, QPCSC *parent );
	~QPCSCReader();

	QByteArray atr() const;
	QString friendlyName() const;
	bool isPinPad() const;
	bool isPresent() const;
	QString name() const;
	int protocol() const;

	bool connect();
	void disconnect( Reset reset = LeaveCard );
	bool reconnect( Reset reset = LeaveCard );
	QByteArray transfer( const QByteArray &cmd );

private:
	QPCSCReaderPrivate *d;
};
