/*
 * QEstEidCommon
 *
 * Copyright (C) 2011-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2011-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include <QtCore/QLibrary>

#ifdef Q_OS_WIN
#include <winsock2.h>
#include <winscard.h>
#else
#include <PCSC/wintypes.h>
#include <PCSC/winscard.h>
#include <arpa/inet.h>
#endif

#ifndef CM_IOCTL_GET_FEATURE_REQUEST
#ifdef Q_CC_MSVC
typedef unsigned __int8 uint8_t;
typedef unsigned __int16 uint16_t;
typedef unsigned __int32 uint32_t;
#else
#include <stdint.h>
#endif
#include "internal-pcsc22.h"
#endif

#ifdef Q_CC_MSVC
#  define SUFFIX "A"
#  undef SCARD_READERSTATE
#  define SCARD_READERSTATE SCARD_READERSTATEA
#else
#  define WINAPI
#  define SUFFIX ""
#endif

#ifndef SCARD_ATTR_DEVICE_FRIENDLY_NAME_A
#define SCARD_ATTR_VALUE(Class, Tag) ((((ULONG)(Class)) << 16) | ((ULONG)(Tag)))
#define SCARD_CLASS_SYSTEM     0x7fff
#define SCARD_ATTR_DEVICE_FRIENDLY_NAME_A SCARD_ATTR_VALUE(SCARD_CLASS_SYSTEM, 0x0003)
#define SCARD_ATTR_DEVICE_FRIENDLY_NAME SCARD_ATTR_DEVICE_FRIENDLY_NAME_A
#endif

typedef LONG (WINAPI *sym_SCardEstablishContext)( DWORD scope,
	LPCVOID res1, LPCVOID res2, SCARDCONTEXT *context );
typedef LONG (WINAPI *sym_SCardReleaseContext)( SCARDCONTEXT hContext );
typedef LONG (WINAPI *sym_SCardListReaders)( SCARDCONTEXT hContext,
	LPCSTR mszGroups, LPSTR mszReaders, LPDWORD pcchReaders );
typedef LONG (WINAPI *sym_SCardGetStatusChange)( SCARDCONTEXT hContext,
	DWORD dwTimeout, SCARD_READERSTATE *rgReaderStates, DWORD cReaders );
typedef LONG (WINAPI *sym_SCardConnect)( SCARDCONTEXT hContext, LPCSTR szReader,
	DWORD dwShareMode, DWORD dwPreferredProtocols, LPSCARDHANDLE hCard, LPDWORD pdwActiveProtocol );
typedef LONG (WINAPI *sym_SCardReconnect)( SCARDHANDLE hCard, DWORD dwShareMode,
	DWORD dwPreferredProtocols, DWORD dwInitialization, LPDWORD pdwActiveProtocol );
typedef LONG (WINAPI *sym_SCardDisconnect)( SCARDHANDLE hCard, DWORD dwDisposition );
typedef LONG (WINAPI *sym_SCardControl)( SCARDHANDLE hCard, DWORD dwControlCode, LPCVOID lpInBuffer,
	DWORD nInBufferSize, LPVOID lpOutBuffer, DWORD nOutBufferSize, LPDWORD lpBytesReturned );
typedef LONG (WINAPI *sym_SCardGetAttrib)( SCARDHANDLE hCard, DWORD dwAttrId,
	LPBYTE pbAttr, LPDWORD pcbAttrLen );
typedef LONG (WINAPI *sym_SCardTransmit)( SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
	LPCBYTE pbSendBuffer, DWORD cbSendLength,
	LPSCARD_IO_REQUEST pioRecvPci, LPBYTE pbRecvBuffer, LPDWORD pcbRecvLength );
typedef LONG (WINAPI *sym_SCardBeginTransaction)( SCARDHANDLE hCard );
typedef LONG (WINAPI *sym_SCardEndTransaction)( SCARDHANDLE hCard, DWORD dwDisposition );

class QPCSCPrivate
{
public:
	QPCSCPrivate();

	QLibrary lib;

	SCARDCONTEXT context;
	bool running;

	sym_SCardEstablishContext f_SCardEstablishContext;
	sym_SCardReleaseContext f_SCardReleaseContext;
	sym_SCardListReaders f_SCardListReaders;
	sym_SCardGetStatusChange f_SCardGetStatusChange;
	sym_SCardConnect f_SCardConnect;
	sym_SCardReconnect f_SCardReconnect;
	sym_SCardDisconnect f_SCardDisconnect;
	sym_SCardControl f_SCardControl;
	sym_SCardGetAttrib f_SCardGetAttrib;
	sym_SCardTransmit f_SCardTransmit;
	sym_SCardBeginTransaction f_SCardBeginTransaction;
	sym_SCardEndTransaction f_SCardEndTransaction;
};

class QPCSCReaderPrivate
{
public:
	QPCSCReaderPrivate( QPCSCPrivate *d );

	QByteArray attrib( DWORD id ) const;
	bool updateState();

	QPCSCPrivate *d;
	SCARDHANDLE card;
	DWORD proto;
	SCARD_READERSTATE state;
	QByteArray reader, friendlyName;

	DWORD verify_ioctl, verify_ioctl_start, verify_ioctl_finish;
	DWORD modify_ioctl, modify_ioctl_start, modify_ioctl_finish;
	bool display;
};
