/*
 * QEstEidUtil
 *
 * Copyright (C) 2009-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2009-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "DiagnosticsDialog.h"

#include "MainWindow.h"
#include "QSmartCard.h"

#include <common/Diagnostics.h>

#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <QtGui/QDesktopServices>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>

DiagnosticsDialog::DiagnosticsDialog( QWidget *parent )
:	QDialog( parent )
{
	QApplication::setOverrideCursor( Qt::WaitCursor );
	setupUi( this );
	setAttribute( Qt::WA_DeleteOnClose, true );

	worker = new Diagnostics();
	connect( worker, SIGNAL(update(QString)), diagnosticsText, SLOT(insertHtml(QString)), Qt::QueuedConnection );
#ifdef Q_OS_LINUX
	worker->run();
#else
	worker->start();
#endif
}

DiagnosticsDialog::~DiagnosticsDialog()
{
	worker->wait();
	delete worker;
}

void DiagnosticsDialog::save()
{
	QString filename = QFileDialog::getSaveFileName( this, tr("Save as"),
		QDesktopServices::storageLocation( QDesktopServices::DocumentsLocation ) + "/diagnostics.txt",
		tr("Text files (*.txt)") );
	if( filename.isEmpty() )
		return;
	QFile f( filename );
	if( !f.open( QIODevice::WriteOnly ) || !f.write( diagnosticsText->toPlainText().toUtf8() ) )
		QMessageBox::warning( this, tr("Error occurred"), tr("Failed write to file!") );
}
